import matrix, scene, ip, tui
from random import uniform

class CubeGeometry(scene.Geometry):
    def generate(self):
        v0 = scene.Vertex((-0.5, -0.5, -0.5), (0,1))
        v1 = scene.Vertex(( 0.5, -0.5, -0.5), (0,0))
        v2 = scene.Vertex(( 0.5,  0.5, -0.5), (1,0))
        v3 = scene.Vertex((-0.5,  0.5, -0.5), (1,1))
        self.quad(v1, v0, v3, v2)
        
        v0 = scene.Vertex((-0.5, -0.5,  0.5), (0,0))
        v1 = scene.Vertex(( 0.5, -0.5,  0.5), (0,1))
        v2 = scene.Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = scene.Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = scene.Vertex((-0.5,  0.5, -0.5), (0,0))
        v1 = scene.Vertex((-0.5, -0.5, -0.5), (0,1))
        v2 = scene.Vertex((-0.5, -0.5,  0.5), (1,1))
        v3 = scene.Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = scene.Vertex(( 0.5, -0.5, -0.5), (0,0))
        v1 = scene.Vertex(( 0.5,  0.5, -0.5), (0,1))
        v2 = scene.Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = scene.Vertex(( 0.5, -0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = scene.Vertex(( 0.5,  0.5, -0.5), (0,0))
        v1 = scene.Vertex((-0.5,  0.5, -0.5), (0,1))
        v2 = scene.Vertex((-0.5,  0.5,  0.5), (1,1))
        v3 = scene.Vertex(( 0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = scene.Vertex((-0.5, -0.5, -0.5), (0,0))
        v1 = scene.Vertex(( 0.5, -0.5, -0.5), (0,1))
        v2 = scene.Vertex(( 0.5, -0.5,  0.5), (1,0))
        v3 = scene.Vertex((-0.5, -0.5,  0.5), (1,1))
        self.quad(v0, v1, v2, v3)
        
        return 0

class PlaneGeometry(scene.Geometry):
    def generate(self):
        v0 = scene.Vertex((-1, 0, -1), (0,1))
        v1 = scene.Vertex(( 1, 0, -1), (0,0))
        v2 = scene.Vertex(( 1, 0,  1), (1,0))
        v3 = scene.Vertex((-1, 0,  1), (1,1))
        self.quad(v1, v0, v3, v2)
        return 0
    
class DiffuseMaterial(scene.Material):
    def __init__(self, r, g, b, a=1):
        scene.Material.__init__(self)
        self.Kd = (r, g, b)
        self.opacity = a
        
    def shade(self, shader):
        c = 0
        for light in shader.light_list:
            a = shader.N.dot(-light.L)
            if a > 0:
                c = c + a
        if c < 0.1: c = 0.3
        kd = shader.Kd
        return (c * kd.r, c * kd.g, c * kd.b, self.opacity)

class DisplacementMaterial(scene.Material):
    def __init__(self, scale=1):
        scene.Material.__init__(self)
        self.scale = scale

    def shade(self, shader):
        p = shader.P + self.scale * shader.D.r * shader.N
        return (p.x, p.y, p.z, 1)
    
class MapMaterial(scene.Material):
    def __init__(self, map_name):
        scene.Material.__init__(self)
        self.image = scene.image.Image()
        d = self.image.read(map_name)
        if d < 0:
            print 'Cannot read texture map ', map_name, d
        else:
            print 'Read texture map ', map_name, d
        
    def shade(self, shader):
        c = self.image.get_color(shader.u, shader.v)
        return (c[0], c[1], c[2], 1)

class DirectLight:
    def __init__(self, C=(1,1,1), L=(0,-1,0)):
        self.L = L
        self.C = C

    def illuminate(self, shader):
        shader.L = self.L
        return (self.C[0], self.C[1], self.C[2])
    
    
def grid(view, n):
    map = MapMaterial("brush.ppm")
    material = DiffuseMaterial(0.3, 0.5, 0.2)
    material.Kd = map
    for x in range(-n, n):
        for y in range(-n,n):
            for z in range(0,n):
                cube = CubeGeometry()
                s = uniform(0.3, 1)
                cube.l2w = matrix.Matrix()
                cube.l2w.set_scale(s, s, s)
                cube.l2w.rotate(uniform(0,360), uniform(0,360), uniform(0,360))
                cube.l2w.translate(x*3, y*3, z*3)
                view.add_geometry(cube, material, 3)
    
def twocubes(view):
    material = DiffuseMaterial(0.5, 0.2, 0.3)
    map = MapMaterial("texture.ppm")
    material.Kd = map

    cube = CubeGeometry()
    cube.l2w = matrix.Matrix()
    cube.l2w.set_translate(0, 0, -1)
    cube.l2w.lookat(3,4,5)
    cube.l2w_motion = matrix.Matrix()
    cube.l2w_motion.set_translate(0, 1, 1)
    cube.l2w_motion.lookat(3,4,5)
    cube.l2w_motion.rotate(0, 15, 0)
    view.add_geometry(cube, material, 3)

    cube = CubeGeometry()
    cube.l2w = matrix.Matrix()
    cube.l2w.set_translate(2, -1, 1)
    cube.l2w.lookat(5,3,7)
    view.add_geometry(cube, material, 3)



def displaced_plane(view):
    plane = PlaneGeometry()
    plane.l2w = matrix.Matrix()
    plane.l2w.set_scale(3, 3, 3)
    plane.l2w.rotate(-30, 0, 0)
    material = DiffuseMaterial(0.7, 0.6, 0.3)
    displacement = DisplacementMaterial(1)
    map = MapMaterial("noise.ppm")
    displacement.D = map
    view.add_geometry(plane, material, 1, displacement)

def transparency(view):
    plane = PlaneGeometry()
    plane.l2w = matrix.Matrix()
    plane.l2w.set_scale(3, 3, 3)
    plane.l2w.rotate(-90, 0, 0)
    material = DiffuseMaterial(0.5, 0.9, 0.5, 0.5)
    view.add_geometry(plane, material, 3)

    material = DiffuseMaterial(0.5, 0.2, 0.3)
    map = MapMaterial("texture.ppm")
    material.Kd = map
    cube = CubeGeometry()
    cube.l2w = matrix.Matrix()
    cube.l2w.set_translate(0, 1, 5)
    cube.l2w.lookat(5,3,7)
    view.add_geometry(cube, material, 3)
    
def run():
    view = scene.ZBufferView()
    view.set_format("1.85")
    view.scale_viewport(3)
    view.add_light(DirectLight(C=(1,1,1)))
    view.add_light(DirectLight(C=(1,1,1),  L=(0,0,-1)))
    view.set_samples(3)
    view.set_filter("gaussian", 2)
    render = view.get_image()

    imv = tui.ImageViewer(render)
    
#    transparency(view)
#    grid(view, 15)
    displaced_plane(view)
#    twocubes(view)
#    view.set_dof(4, 0.1, 50, 5)
    view.render()

    # Composite render over a gradient background with dropshadowed text
#    render.write("render.ppm")
    ip.match_viewport(render)
    foreground = ip.image(render)
    background = ip.fill((1, 1, 1, 1, 1))
#    background = ip.gradient((1.0, 1.0, 1.0, 1, 1), (1.0, 1.0, 1.0, 1, 1),
#                             (0.7, 0.7, 0.7, 1, 1), (0.7, 0.7, 0.7, 1, 1))
    comp = ip.matte(foreground, background).get_image()
    ft = ip.Font("/usr/share/fonts/default/TrueType/helr____.ttf")
    ft.set_size(16)
    text = "Cubes (c) Daniel Wexler, www.flarg.com"
    ft.draw_string(comp, 20, 10, text, 0.3, 0.3, 0.3)
    ft.draw_string(comp, 18, 12, text, 0.3,0.4,0.8)
    comp.write("comp.ppm")

